 %-----------------------------------------------------------------
% Linear Programming Algoriths for Sparse Filter Design
%
%   Authors     Ludwig Mohr, Peter Reinprecht
%   Date        Januar 2012
%   Course      Advanced Signalprocessing 1, Seminar - WS 2011/12
%
%-----------------------------------------------------------------

clear all
close all

%% LP Filter
filtername = 'LP';
load('LP.mat');
h = Num;
N = length(h)-1;

% figure
% freqz(h)

fs = 48e3;
fpass = 10e3;
fstop = 11e3;

wp_range = [0, fpass/(fs/2)]*pi;
wp_threshold = [1];
ws_range = [fstop/(fs/2), 1]*pi;
ws_threshold = -40:5:-20;

[hval, hfreq] = freqz(h);
stoprange = find(hfreq > ws_range(1) & hfreq < ws_range(2));
deltaV = [];


for k = 1:length(ws_threshold);    
	[h_sm1, delta_sm1, Z_sm1] = sparse_minimum_1_norm(h, wp_range, wp_threshold, ws_range, ws_threshold(k));
	deltaV(k, 1) = delta_sm1(length(delta_sm1));
    [hval, hfreq] = freqz(h_sm1);
    deltaV(k, 2) = max(20*log10(abs(hval(stoprange)))); 
    deltaV(k, 3) = length(Z_sm1); 
end

delta_sm1 = deltaV;

wp_range = [0, fpass/(fs/2)]*pi;
wp_threshold = [1];
ws_range = [fstop/(fs/2), 1]*pi;
ws_threshold = -30;

tic
[h_sc, delta_sc, Z_sc, error_dB_sc] = sparse_smallest_coefficient(h, wp_range, wp_threshold, ws_range, ws_threshold);
t_sc = toc;
tic
[h_sco, delta_sco, Z_sco, error_dB_sco] = sparse_smallest_coefficient_opt(h, wp_range, wp_threshold, ws_range, ws_threshold);
t_sco = toc;
tic
[h_si, delta_si, Z_si, error_dB_si] = sparse_smallest_increase(h, wp_range, wp_threshold, ws_range, ws_threshold);
t_si = toc;

save(strcat('sparse_opt_result_all-dB'), ...
    'h_sc', 'delta_sc', 'Z_sc', 'error_dB_sc', 't_sc',     ... % smallest coefficient
    'h_sco', 'delta_sco', 'Z_sco', 'error_dB_sco', 't_sco', ... % smallest coefficient opt
    'h_si', 'delta_si', 'Z_si', 'error_dB_si', 't_si',     ... % smallest increase
    'h_sm1', 'delta_sm1', 'Z_sm1');    % minimum 1 norm

break;

for k = 1:length(ws_threshold);
    
    test_sparse_opt(strcat(filtername, '_', int2str(ws_threshold(k)), 'dB'), ...
        h, wp_range, wp_threshold, ws_range, ws_threshold(k));

end

%% HP Filter
filtername = 'HP';
load('HP.mat');
h = Num;
N = length(h)-1;

% figure
% freqz(h)

fs = 48e3;
fpass = 11e3;
fstop = 10e3;

wp_range = [fpass/(fs/2), 1]*pi;
wp_threshold = [1];
ws_range = [0, fstop/(fs/2)]*pi;
ws_threshold = -40:5:-20;

for k = 1:length(ws_threshold);
    
    test_sparse_opt(strcat(filtername, '_', int2str(ws_threshold(k)), 'dB'), ...
        h, wp_range, wp_threshold, ws_range, ws_threshold(k));

end

break;

%% BP Filter
filtername = 'BP';
load('BP.mat');
h = Num;
N = length(h)-1;

% figure
% freqz(h);

fs = 48e3;
fpass_l = 9e3;
fstop_l = 7e3;
fpass_u = 14e3;
fstop_u = 16e3;

wp_range = [fpass_l/(fs/2), fpass_u/(fs/2)]*pi;
wp_threshold = [1];
ws_range = [0, fstop_l/(fs/2); fstop_u/(fs/2), 1]*pi;
ws_threshold = [(-70:5:-50)', (-70:5:-50)'];

for k = 1:length(ws_threshold);
    
    test_sparse_opt(strcat(filtername, '_', int2str(ws_threshold(k,1)), 'dB'), ...
        h, wp_range, wp_threshold, ws_range, ws_threshold(k,:));

end


%% FIN

